import os
import urllib.parse

from datacontract.data_contract import DataContractSpecification


def _normalize_path(path: str) -> str:
    parsed = urllib.parse.urlparse(path)

    if not parsed.scheme or parsed.scheme == "file":
        full_path = os.path.abspath(os.path.expanduser(path))
        return f"file://{full_path}"
    else:
        return path


def get_server_information(
    data_contract_specification: DataContractSpecification,
    server_name: str | None,
    asset_name: str,
) -> dict[str, str]:
    server = data_contract_specification.servers.get(server_name)
    if not server:
        return {}

    server_information = {}
    match server.type:
        case "azure":
            server_information["dagster/uri"] = server.location
        case "databricks":
            server_information["dagster/uri"] = server.host
            server_information["dagster/table_name"] = (
                f"{server.catalog}.{server.schema}.{asset_name}"
            )
        case "kafka":
            server_information["dagster/uri"] = server.host
        case "kinesis":
            server_information = {}
        case "local":
            server_information["dagster/uri"] = _normalize_path(server.path)
        case "oracle":
            server_information["dagster/uri"] = f"{server.host}:{server.port}"
        case "postgres":
            server_information["dagster/uri"] = f"{server.host}:{server.port}"
            server_information["dagster/table_name"] = (
                f"{server.database}.{server.schema}.{asset_name}"
            )
        case "pubsub":
            server_information = {}
        case "redshift":
            server_information["dagster/uri"] = server.endpoint
            server_information["dagster/table_name"] = (
                f"{server.database}.{server.schema}.{asset_name}"
            )
        case "s3":
            server_information["dagster/uri"] = server.location
        case "sftp":
            server_information["dagster/uri"] = server.location
        case "snowflake":
            server_information["dagster/table_name"] = (
                f"{server.database}.{server.schema}.{asset_name}"
            )
        case "sqlserver":
            server_information["dagster/table_name"] = (
                f"{server.database}.{server.schema}.{asset_name}"
            )
        case "trino":
            server_information["dagster/uri"] = f"{server.host}:{server.port}"
            server_information["dagster/table_name"] = (
                f"{server.catalog}.{server.schema}.{asset_name}"
            )
        case _:
            server_information = {}

    return server_information



from dagster_datacontract.metadata.server_information import get_server_information
from dagster_datacontract.metadata.table_colums import (
    get_column_lineage,
    get_table_column,
)

__all__ = ["get_table_column", "get_column_lineage", "get_server_information"]



import re

from loguru import logger


def get_tags(
    tags_list: list[str] | None,
) -> dict[str, str]:
    """Parse and validate a list of string tags into a dictionary format.

    Each tag in the input list should be in the form "key:value" or simply "key".
    - Keys must match the pattern: ^[\w.-]{1,63}$
    - Values (if provided) must match the pattern: ^[\w.-]{0,63}$

    Invalid tags (those that do not match the expected format) will be ignored,
    and a warning will be logged.

    More information about Dagster tags:
    https://docs.dagster.io/guides/build/assets/metadata-and-tags/tags

    Args:
        tags_list (list[str] | None): A list of tags as strings. Each tag may be
            formatted as "key:value" or just "key". If None, an empty dict is returned.

    Returns:
        dict[str, str]: A dictionary of validated tags, where keys are tag names
        and values are tag values (empty string if not provided).
    """
    key_pattern = re.compile(r"^[\w.-]{1,63}$")
    val_pattern = re.compile(r"^[\w.-]{0,63}$")

    tags = {}

    for item in tags_list:
        if ":" in item:
            key, val = map(str.strip, item.split(":", 1))
        else:
            key, val = item.strip(), ""

        if key_pattern.match(key) and val_pattern.match(val):
            tags[key] = val
        else:
            logger.warning(f"Ignoring invalid tag: {item}")

    return tags



import textwrap
from typing import Any

from datacontract.data_contract import DataContractSpecification


def get_description(
    asset_name: str,
    data_contract_specification: DataContractSpecification,
    config: dict[str, Any] | None = None,
    separator: str = "\n",
) -> str | None:
    """Load and return a formatted description string based on the data contract specification.

    This method composes a description by pulling text from different parts
    of the data contract specification (e.g., model and info descriptions),
    joining them using the specified separator.

    Args:
        config (dict[str, Any] | None, optional): A configuration dictionary
            specifying the order in which to concatenate the description parts.
            Defaults to `{"order": ["model", "info"]}`.
        separator (str, optional): A string used to separate different parts
            of the description. Defaults to a newline character (`"\n"`).

    Returns:
        str | None: A single string combining the specified description parts
        if available, otherwise `None`.
    """
    default_config = {"order": ["model", "info"]}

    configuration = default_config | (config or {})

    descriptions = {
        "model": data_contract_specification.models.get(asset_name).description,
        "info": data_contract_specification.info.description,
    }

    parts = []
    for key in configuration["order"]:
        desc = descriptions.get(key).replace("\n", f"{separator}\n")
        if desc:
            parts.append(textwrap.dedent(desc))

    if parts:
        return f"{separator}\n".join(parts)

    return None
